import { Toml } from '../../types';

export function generateWorkspaceCargoToml(rootPath: string): Toml {
    return `
        # This code is automatically generated by Azle

        [workspace]
        members = [
            "${rootPath}",
            "${rootPath}/azle_generate"
        ]

        [profile.release]
        lto = true
        opt-level = 'z'
    `;
}

export function generateLibCargoToml(canisterName: string): Toml {
    return `
        # This code is automatically generated by Azle

        [package]
        name = "${canisterName}"
        version = "0.0.0"
        edition = "2018"

        [lib]
        crate-type = ["cdylib"]

        [dependencies]
        ic-cdk = "0.5.2"
        ic-cdk-macros = "0.5.2"
        candid = "0.7.16"
        boa_engine = { git = "https://github.com/demergent-labs/boa", branch = "date" } # This is a fork of call_stack
        # boa_engine = { git = "https://github.com/demergent-labs/boa", branch = "call_stack" } # This is a fork of vec_u8_into_typed_array
        # boa_engine = { git = "https://github.com/demergent-labs/boa", branch = "vec_u8_into_typed_array" } # This is a fork of js-bigint-128
        # boa_engine = { git = "https://github.com/demergent-labs/boa", branch = "js-bigint-128" } # This has a small necessary change that another person has opened a pull request to fix
        # boa_engine = { git = "https://github.com/boa-dev/boa", rev = "f3db18fc5468576d2c6df31b259890c1d43d3607" } # This is the last main git commit we were at
        # boa_engine = { path = "../../../../../../boa/boa_engine" }
        getrandom = { version = "0.2.3", features = ["custom"] }
        serde = "1.0.137"
        azle-vm-value-derive = { path = "./azle_vm_value_derive" }
        # azle_generate_macro = { path = "./azle_generate_macro" }
        async-recursion = "1.0.0"
    `;
}

export function generateWorkspaceCargoLock(): Toml {
    return `
        # This file is automatically @generated by Cargo.
        # It is not intended for manual editing.
        version = 3

        [[package]]
        name = "Inflector"
        version = "0.11.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "fe438c63458706e03479442743baae6c88256498e6431708f6dfc520a26515d3"
        dependencies = [
        "lazy_static",
        "regex",
        ]

        [[package]]
        name = "ahash"
        version = "0.7.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "fcb51a0695d8f838b1ee009b3fbf66bda078cd64590202a864a8f3e8c4315c47"
        dependencies = [
        "getrandom",
        "once_cell",
        "version_check",
        ]

        [[package]]
        name = "aho-corasick"
        version = "0.7.18"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1e37cfd5e7657ada45f742d6e99ca5788580b5c529dc78faf11ece6dc702656f"
        dependencies = [
        "memchr",
        ]

        [[package]]
        name = "anyhow"
        version = "1.0.57"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "08f9b8508dccb7687a1d6c4ce66b2b0ecef467c94667de27d8d7fe1f8d2a9cdc"

        [[package]]
        name = "arrayvec"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "23b62fc65de8e4e7f52534fb52b0f3ed04746ae267519eef2a83941e8085068b"

        [[package]]
        name = "ascii-canvas"
        version = "3.0.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "8824ecca2e851cec16968d54a01dd372ef8f95b244fb84b84e70128be347c3c6"
        dependencies = [
        "term",
        ]

        [[package]]
        name = "ast_node"
        version = "0.8.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0a614981a880a40522cf6fbe8b1a8365eb253655939f812a9db03e8ba4e2cb1f"
        dependencies = [
        "darling",
        "pmutil",
        "proc-macro2",
        "quote",
        "swc_macros_common",
        "syn",
        ]

        [[package]]
        name = "async-recursion"
        version = "1.0.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2cda8f4bcc10624c4e85bc66b3f452cca98cfa5ca002dc83a16aad2367641bea"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "atty"
        version = "0.2.14"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d9b39be18770d11421cdb1b9947a45dd3f37e93092cbf377614828a319d5fee8"
        dependencies = [
        "hermit-abi",
        "libc",
        "winapi",
        ]

        [[package]]
        name = "autocfg"
        version = "1.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d468802bab17cbc0cc575e9b053f41e72aa36bfa6b7f55e3529ffa43161b97fa"

        [[package]]
        name = "azle-vm-value-derive"
        version = "0.0.0"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "azle_generate"
        version = "0.0.0"
        dependencies = [
        "derive-syn-parse",
        "proc-macro2",
        "quote",
        "swc_common",
        "swc_ecma_ast",
        "swc_ecma_parser",
        "syn",
        "uuid",
        ]

        [[package]]
        name = "beef"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3a8241f3ebb85c056b509d4327ad0358fbbba6ffb340bf388f26350aeda225b1"

        [[package]]
        name = "better_scoped_tls"
        version = "0.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b73e8ecdec39e98aa3b19e8cd0b8ed8f77ccb86a6b0b2dc7cd86d105438a2123"
        dependencies = [
        "scoped-tls",
        ]

        [[package]]
        name = "binread"
        version = "2.2.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "16598dfc8e6578e9b597d9910ba2e73618385dc9f4b1d43dd92c349d6be6418f"
        dependencies = [
        "binread_derive",
        "lazy_static",
        "rustversion",
        ]

        [[package]]
        name = "binread_derive"
        version = "2.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1d9672209df1714ee804b1f4d4f68c8eb2a90b1f7a07acf472f88ce198ef1fed"
        dependencies = [
        "either",
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "bit-set"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6e11e16035ea35e4e5997b393eacbf6f63983188f7a2ad25bfb13465f5ad59de"
        dependencies = [
        "bit-vec",
        ]

        [[package]]
        name = "bit-vec"
        version = "0.6.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "349f9b6a179ed607305526ca489b34ad0a41aed5f7980fa90eb03160b69598fb"

        [[package]]
        name = "bitflags"
        version = "1.3.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "bef38d45163c2f1dde094a7dfd33ccf595c92905c8f8f4fdc18d06fb1037718a"

        [[package]]
        name = "block-buffer"
        version = "0.10.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0bf7fe51849ea569fd452f37822f606a5cabb684dc918707a0193fd4664ff324"
        dependencies = [
        "generic-array",
        ]

        [[package]]
        name = "boa_engine"
        version = "0.14.0"
        source = "git+https://github.com/demergent-labs/boa?branch=date#873a53abd548c8ab2f0944294a8e4aa1fa4c94f4"
        dependencies = [
        "bitflags",
        "boa_gc",
        "boa_interner",
        "boa_profiler",
        "boa_unicode",
        "chrono",
        "dyn-clone",
        "fast-float",
        "gc",
        "ic-cdk",
        "icu",
        "indexmap",
        "num-bigint",
        "num-integer",
        "num-traits",
        "once_cell",
        "rand",
        "regress",
        "rustc-hash",
        "ryu-js",
        "serde",
        "serde_json",
        "tap",
        "unicode-normalization",
        ]

        [[package]]
        name = "boa_gc"
        version = "0.14.0"
        source = "git+https://github.com/demergent-labs/boa?branch=date#873a53abd548c8ab2f0944294a8e4aa1fa4c94f4"
        dependencies = [
        "gc",
        ]

        [[package]]
        name = "boa_interner"
        version = "0.14.0"
        source = "git+https://github.com/demergent-labs/boa?branch=date#873a53abd548c8ab2f0944294a8e4aa1fa4c94f4"
        dependencies = [
        "gc",
        "string-interner",
        ]

        [[package]]
        name = "boa_profiler"
        version = "0.14.0"
        source = "git+https://github.com/demergent-labs/boa?branch=date#873a53abd548c8ab2f0944294a8e4aa1fa4c94f4"

        [[package]]
        name = "boa_unicode"
        version = "0.14.0"
        source = "git+https://github.com/demergent-labs/boa?branch=date#873a53abd548c8ab2f0944294a8e4aa1fa4c94f4"
        dependencies = [
        "unicode-general-category",
        ]

        [[package]]
        name = "byteorder"
        version = "1.4.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "14c189c53d098945499cdfa7ecc63567cf3886b3332b312a5b4585d8d3a6a610"

        [[package]]
        name = "candid"
        version = "0.7.16"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b7e4287605536419ac6ce9d76d3aa5733677ca8c7c2891079b2c71a345cc17a3"
        dependencies = [
        "anyhow",
        "binread",
        "byteorder",
        "candid_derive",
        "codespan-reporting",
        "hex",
        "ic-types",
        "lalrpop",
        "lalrpop-util",
        "leb128",
        "logos",
        "num-bigint",
        "num-traits",
        "num_enum",
        "paste",
        "pretty",
        "serde",
        "serde_bytes",
        "thiserror",
        ]

        [[package]]
        name = "candid_derive"
        version = "0.4.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2e02c03c4d547674a3f3f3109538fb49871fbe636216daa019f06a62faca9061"
        dependencies = [
        "lazy_static",
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "cfg-if"
        version = "1.0.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "baf1de4339761588bc0619e3cbc0120ee582ebb74b53b4efbf79117bd2da40fd"

        [[package]]
        name = "chrono"
        version = "0.4.19"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "670ad68c9088c2a963aaa298cb369688cf3f9465ce5e2d4ca10e6e0098a1ce73"
        dependencies = [
        "libc",
        "num-integer",
        "num-traits",
        "time",
        "winapi",
        ]

        [[package]]
        name = "codespan-reporting"
        version = "0.11.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3538270d33cc669650c4b093848450d380def10c331d38c768e34cac80576e6e"
        dependencies = [
        "termcolor",
        "unicode-width",
        ]

        [[package]]
        name = "cpufeatures"
        version = "0.2.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "59a6001667ab124aebae2a495118e11d30984c3a653e99d86d58971708cf5e4b"
        dependencies = [
        "libc",
        ]

        [[package]]
        name = "crc32fast"
        version = "1.3.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b540bd8bc810d3885c6ea91e2018302f68baba2129ab3e88f32389ee9370880d"
        dependencies = [
        "cfg-if",
        ]

        [[package]]
        name = "crunchy"
        version = "0.2.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "7a81dae078cea95a014a339291cec439d2f232ebe854a9d672b796c6afafa9b7"

        [[package]]
        name = "crypto-common"
        version = "0.1.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1bfb12502f3fc46cca1bb51ac28df9d618d813cdc3d2f25b9fe775a34af26bb3"
        dependencies = [
        "generic-array",
        "typenum",
        ]

        [[package]]
        name = "darling"
        version = "0.13.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a01d95850c592940db9b8194bc39f4bc0e89dee5c4265e4b1807c34a9aba453c"
        dependencies = [
        "darling_core",
        "darling_macro",
        ]

        [[package]]
        name = "darling_core"
        version = "0.13.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "859d65a907b6852c9361e3185c862aae7fafd2887876799fa55f5f99dc40d610"
        dependencies = [
        "fnv",
        "ident_case",
        "proc-macro2",
        "quote",
        "strsim",
        "syn",
        ]

        [[package]]
        name = "darling_macro"
        version = "0.13.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "9c972679f83bdf9c42bd905396b6c3588a843a17f0f16dfcfa3e2c5d57441835"
        dependencies = [
        "darling_core",
        "quote",
        "syn",
        ]

        [[package]]
        name = "data-encoding"
        version = "2.3.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3ee2393c4a91429dffb4bedf19f4d6abf27d8a732c8ce4980305d782e5426d57"

        [[package]]
        name = "debug_unreachable"
        version = "0.1.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "9a032eac705ca39214d169f83e3d3da290af06d8d1d344d1baad2fd002dca4b3"
        dependencies = [
        "unreachable",
        ]

        [[package]]
        name = "derive-syn-parse"
        version = "0.1.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e79116f119dd1dba1abf1f3405f03b9b0e79a27a3883864bfebded8a3dc768cd"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "diff"
        version = "0.1.12"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0e25ea47919b1560c4e3b7fe0aaab9becf5b84a10325ddf7db0f0ba5e1026499"

        [[package]]
        name = "digest"
        version = "0.10.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f2fb860ca6fafa5552fb6d0e816a69c8e49f0908bf524e30a90d97c85892d506"
        dependencies = [
        "block-buffer",
        "crypto-common",
        ]

        [[package]]
        name = "dirs-next"
        version = "2.0.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b98cf8ebf19c3d1b223e151f99a4f9f0690dca41414773390fc824184ac833e1"
        dependencies = [
        "cfg-if",
        "dirs-sys-next",
        ]

        [[package]]
        name = "dirs-sys-next"
        version = "0.1.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "4ebda144c4fe02d1f7ea1a7d9641b6fc6b580adcfa024ae48797ecdeb6825b4d"
        dependencies = [
        "libc",
        "redox_users",
        "winapi",
        ]

        [[package]]
        name = "displaydoc"
        version = "0.2.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3bf95dc3f046b9da4f2d51833c0d3547d8564ef6910f5c1ed130306a75b92886"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "dyn-clone"
        version = "1.0.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "140206b78fb2bc3edbcfc9b5ccbd0b30699cfe8d348b8b31b330e47df5291a5a"

        [[package]]
        name = "either"
        version = "1.6.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e78d4f1cc4ae33bbfc157ed5d5a5ef3bc29227303d595861deb238fcec4e9457"

        [[package]]
        name = "ena"
        version = "0.14.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d7402b94a93c24e742487327a7cd839dc9d36fec9de9fb25b09f2dae459f36c3"
        dependencies = [
        "log",
        ]

        [[package]]
        name = "enum_kind"
        version = "0.2.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "78b940da354ae81ef0926c5eaa428207b8f4f091d3956c891dfbd124162bed99"
        dependencies = [
        "pmutil",
        "proc-macro2",
        "swc_macros_common",
        "syn",
        ]

        [[package]]
        name = "fast-float"
        version = "0.2.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "95765f67b4b18863968b4a1bd5bb576f732b29a4a28c7cd84c09fa3e2875f33c"

        [[package]]
        name = "fixed_decimal"
        version = "0.2.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "857766d9808ad8f9033fe4e6499fcc6504ae485cc99ef2f3a98b4850c23b895a"
        dependencies = [
        "displaydoc",
        "smallvec",
        "static_assertions",
        "writeable",
        ]

        [[package]]
        name = "fixedbitset"
        version = "0.4.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "279fb028e20b3c4c320317955b77c5e0c9701f05a1d309905d6fc702cdc5053e"

        [[package]]
        name = "fnv"
        version = "1.0.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3f9eec918d3f24069decb9af1554cad7c880e2da24a9afd88aca000531ab82c1"

        [[package]]
        name = "form_urlencoded"
        version = "1.0.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5fc25a87fa4fd2094bffb06925852034d90a17f0d1e05197d4956d3555752191"
        dependencies = [
        "matches",
        "percent-encoding",
        ]

        [[package]]
        name = "from_variant"
        version = "0.1.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f0981e470d2ab9f643df3921d54f1952ea100c39fdb6a3fdc820e20d2291df6c"
        dependencies = [
        "pmutil",
        "proc-macro2",
        "swc_macros_common",
        "syn",
        ]

        [[package]]
        name = "gc"
        version = "0.4.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3edaac0f5832202ebc99520cb77c932248010c4645d20be1dc62d6579f5b3752"
        dependencies = [
        "gc_derive",
        ]

        [[package]]
        name = "gc_derive"
        version = "0.4.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "60df8444f094ff7885631d80e78eb7d88c3c2361a98daaabb06256e4500db941"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        "synstructure",
        ]

        [[package]]
        name = "generic-array"
        version = "0.14.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "fd48d33ec7f05fbfa152300fdad764757cbded343c1aa1cff2fbaf4134851803"
        dependencies = [
        "typenum",
        "version_check",
        ]

        [[package]]
        name = "getrandom"
        version = "0.2.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "4eb1a864a501629691edf6c15a593b7a51eebaa1e8468e9ddc623de7c9b58ec6"
        dependencies = [
        "cfg-if",
        "libc",
        "wasi 0.11.0+wasi-snapshot-preview1",
        ]

        [[package]]
        name = "hashbrown"
        version = "0.11.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "ab5ef0d4909ef3724cc8cce6ccc8572c5c817592e9285f5464f8e86f8bd3726e"
        dependencies = [
        "ahash",
        ]

        [[package]]
        name = "hashbrown"
        version = "0.12.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "db0d4cf898abf0081f964436dc980e96670a0f36863e4b83aaacdb65c9d7ccc3"

        [[package]]
        name = "hermit-abi"
        version = "0.1.19"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "62b467343b94ba476dcb2500d242dadbb39557df889310ac77c5d99100aaac33"
        dependencies = [
        "libc",
        ]

        [[package]]
        name = "hex"
        version = "0.4.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "7f24254aa9a54b5c858eaee2f5bccdb46aaf0e486a595ed5fd8f86ba55232a70"

        [[package]]
        name = "ic-cdk"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e471fa2e2464d4f9d676b81d33ae0d1cd190981166c53bbe4d536f13da281fc6"
        dependencies = [
        "candid",
        "cfg-if",
        "serde",
        ]

        [[package]]
        name = "ic-cdk-macros"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "fb6d2c14db28f427154a52088077419edf1848e2bbd0f527dee3150e592a0863"
        dependencies = [
        "candid",
        "ic-cdk",
        "proc-macro2",
        "quote",
        "serde",
        "serde_tokenstream",
        "syn",
        ]

        [[package]]
        name = "ic-types"
        version = "0.4.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0a7d9569a4c91d5c9c202c1666f799a9761bab70a2b59464ee78c45b005e47b8"
        dependencies = [
        "crc32fast",
        "data-encoding",
        "hex",
        "serde",
        "serde_bytes",
        "sha2",
        "thiserror",
        ]

        [[package]]
        name = "icu"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "904a80ce690d14859a54e02d803eefba4ecd021882b8ab0492e455951d4df9a5"
        dependencies = [
        "fixed_decimal",
        "icu_calendar",
        "icu_datetime",
        "icu_decimal",
        "icu_locale_canonicalizer",
        "icu_locid",
        "icu_locid_macros",
        "icu_plurals",
        "icu_properties",
        ]

        [[package]]
        name = "icu_calendar"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "07549f5d96acacdee05054133bd74ab1179211185ebe886f953fe1a0dd634096"
        dependencies = [
        "displaydoc",
        "icu_provider",
        "litemap",
        "tinystr",
        ]

        [[package]]
        name = "icu_codepointtrie"
        version = "0.3.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6b54c854632ef55b92a825f9f26038a3be7f72116036aee05c522b6a65323beb"
        dependencies = [
        "displaydoc",
        "serde",
        "yoke",
        "zerovec",
        ]

        [[package]]
        name = "icu_datetime"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6cd9ae537546879a4e57a09d3fc7f0ae48a1c2d8f605ce14536b04f99afe5f3e"
        dependencies = [
        "displaydoc",
        "either",
        "icu_calendar",
        "icu_locid",
        "icu_plurals",
        "icu_provider",
        "litemap",
        "num_enum",
        "serde",
        "smallvec",
        "tinystr",
        "writeable",
        "zerovec",
        ]

        [[package]]
        name = "icu_decimal"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0bb632b080fd9944616933eb659279228d43801d4c58152e2938fac6f94ecc01"
        dependencies = [
        "displaydoc",
        "fixed_decimal",
        "icu_locid",
        "icu_provider",
        "serde",
        "writeable",
        ]

        [[package]]
        name = "icu_locale_canonicalizer"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "293bbe9de3262a23f0dac731e1c52927f54347d0fd845a1f5aa7de3cc26f3358"
        dependencies = [
        "icu_locid",
        "icu_provider",
        "litemap",
        "serde",
        "tinystr",
        ]

        [[package]]
        name = "icu_locid"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "83a3b8515791232b1be02c21467c123216a0e60fd024f4a74cc119ef5b1af79e"
        dependencies = [
        "displaydoc",
        "serde",
        "tinystr",
        "writeable",
        ]

        [[package]]
        name = "icu_locid_macros"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0c40e3635df35d94a0294b07efadb52b58017a3e16fe86a5ffddfc3e56a36ae1"
        dependencies = [
        "icu_locid",
        "proc-macro-crate",
        "tinystr",
        ]

        [[package]]
        name = "icu_plurals"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "34c03a32a72566b65e6245ad4e9f8e4d470e1decb1fcd10a9cab061bb9fbce92"
        dependencies = [
        "displaydoc",
        "fixed_decimal",
        "icu_locid",
        "icu_provider",
        "num_enum",
        "serde",
        "zerovec",
        ]

        [[package]]
        name = "icu_properties"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3db71ea31fe47694e165989886826e23f9defbae5dbd002e678d756086d1f5c7"
        dependencies = [
        "displaydoc",
        "icu_codepointtrie",
        "icu_provider",
        "icu_uniset",
        "num_enum",
        "serde",
        "zerovec",
        ]

        [[package]]
        name = "icu_provider"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "496b8bc2f7c2fe705191442201ea0db4c86153cabe3f39dbddb95cad3aca6454"
        dependencies = [
        "displaydoc",
        "icu_locid",
        "icu_provider_macros",
        "litemap",
        "serde",
        "tinystr",
        "writeable",
        "yoke",
        ]

        [[package]]
        name = "icu_provider_macros"
        version = "0.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "ea1a9a3f206d9be3e027c3d1448aa8edd690c0aa789d8dca01a17fa76fb79524"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "icu_uniset"
        version = "0.4.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0ce83f199e54f8e0a948d7b737a62a7e4075760d6aef671ef85e52734a05db66"
        dependencies = [
        "displaydoc",
        "litemap",
        "serde",
        "tinystr",
        "yoke",
        "zerovec",
        ]

        [[package]]
        name = "ident_case"
        version = "1.0.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b9e0384b61958566e926dc50660321d12159025e767c18e043daf26b70104c39"

        [[package]]
        name = "idna"
        version = "0.2.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "418a0a6fab821475f634efe3ccc45c013f742efe03d853e8d3355d5cb850ecf8"
        dependencies = [
        "matches",
        "unicode-bidi",
        "unicode-normalization",
        ]

        [[package]]
        name = "indexmap"
        version = "1.9.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6c6392766afd7964e2531940894cffe4bd8d7d17dbc3c1c4857040fd4b33bdb3"
        dependencies = [
        "autocfg",
        "hashbrown 0.12.1",
        ]

        [[package]]
        name = "is-macro"
        version = "0.2.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1c068d4c6b922cd6284c609cfa6dec0e41615c9c5a1a4ba729a970d8daba05fb"
        dependencies = [
        "Inflector",
        "pmutil",
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "itertools"
        version = "0.10.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a9a9d19fa1e79b6215ff29b9d6880b706147f16e9b1dbb1e4e5947b5b02bc5e3"
        dependencies = [
        "either",
        ]

        [[package]]
        name = "itoa"
        version = "1.0.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "112c678d4050afce233f4f2852bb2eb519230b3cf12f33585275537d7e41578d"

        [[package]]
        name = "lalrpop"
        version = "0.19.8"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b30455341b0e18f276fa64540aff54deafb54c589de6aca68659c63dd2d5d823"
        dependencies = [
        "ascii-canvas",
        "atty",
        "bit-set",
        "diff",
        "ena",
        "itertools",
        "lalrpop-util",
        "petgraph",
        "pico-args",
        "regex",
        "regex-syntax",
        "string_cache",
        "term",
        "tiny-keccak",
        "unicode-xid",
        ]

        [[package]]
        name = "lalrpop-util"
        version = "0.19.8"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "bcf796c978e9b4d983414f4caedc9273aa33ee214c5b887bd55fde84c85d2dc4"
        dependencies = [
        "regex",
        ]

        [[package]]
        name = "lazy_static"
        version = "1.4.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e2abad23fbc42b3700f2f279844dc832adb2b2eb069b2df918f455c4e18cc646"

        [[package]]
        name = "leb128"
        version = "0.2.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "884e2677b40cc8c339eaefcb701c32ef1fd2493d71118dc0ca4b6a736c93bd67"

        [[package]]
        name = "lexical"
        version = "6.1.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "c7aefb36fd43fef7003334742cbf77b243fcd36418a1d1bdd480d613a67968f6"
        dependencies = [
        "lexical-core",
        ]

        [[package]]
        name = "lexical-core"
        version = "0.8.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2cde5de06e8d4c2faabc400238f9ae1c74d5412d03a7bd067645ccbc47070e46"
        dependencies = [
        "lexical-parse-float",
        "lexical-parse-integer",
        "lexical-util",
        "lexical-write-float",
        "lexical-write-integer",
        ]

        [[package]]
        name = "lexical-parse-float"
        version = "0.8.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "683b3a5ebd0130b8fb52ba0bdc718cc56815b6a097e28ae5a6997d0ad17dc05f"
        dependencies = [
        "lexical-parse-integer",
        "lexical-util",
        "static_assertions",
        ]

        [[package]]
        name = "lexical-parse-integer"
        version = "0.8.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6d0994485ed0c312f6d965766754ea177d07f9c00c9b82a5ee62ed5b47945ee9"
        dependencies = [
        "lexical-util",
        "static_assertions",
        ]

        [[package]]
        name = "lexical-util"
        version = "0.8.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5255b9ff16ff898710eb9eb63cb39248ea8a5bb036bea8085b1a767ff6c4e3fc"
        dependencies = [
        "static_assertions",
        ]

        [[package]]
        name = "lexical-write-float"
        version = "0.8.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "accabaa1c4581f05a3923d1b4cfd124c329352288b7b9da09e766b0668116862"
        dependencies = [
        "lexical-util",
        "lexical-write-integer",
        "static_assertions",
        ]

        [[package]]
        name = "lexical-write-integer"
        version = "0.8.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e1b6f3d1f4422866b68192d62f77bc5c700bee84f3069f2469d7bc8c77852446"
        dependencies = [
        "lexical-util",
        "static_assertions",
        ]

        [[package]]
        name = "libc"
        version = "0.2.126"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "349d5a591cd28b49e1d1037471617a32ddcda5731b99419008085f72d5a53836"

        [[package]]
        name = "litemap"
        version = "0.3.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5568cf6856c84333bda775ba41b834b20f75d64888067c8c2e26a09154a5730e"
        dependencies = [
        "serde",
        "serde_json",
        ]

        [[package]]
        name = "lock_api"
        version = "0.4.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "327fa5b6a6940e4699ec49a9beae1ea4845c6bab9314e4f84ac68742139d8c53"
        dependencies = [
        "autocfg",
        "scopeguard",
        ]

        [[package]]
        name = "log"
        version = "0.4.17"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "abb12e687cfb44aa40f41fc3978ef76448f9b6038cad6aef4259d3c095a2382e"
        dependencies = [
        "cfg-if",
        ]

        [[package]]
        name = "logos"
        version = "0.12.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "bf8b031682c67a8e3d5446840f9573eb7fe26efe7ec8d195c9ac4c0647c502f1"
        dependencies = [
        "logos-derive",
        ]

        [[package]]
        name = "logos-derive"
        version = "0.12.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a1d849148dbaf9661a6151d1ca82b13bb4c4c128146a88d05253b38d4e2f496c"
        dependencies = [
        "beef",
        "fnv",
        "proc-macro2",
        "quote",
        "regex-syntax",
        "syn",
        ]

        [[package]]
        name = "matches"
        version = "0.1.9"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a3e378b66a060d48947b590737b30a1be76706c8dd7b8ba0f2fe3989c68a853f"

        [[package]]
        name = "memchr"
        version = "2.5.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2dffe52ecf27772e601905b7522cb4ef790d2cc203488bbd0e2fe85fcb74566d"

        [[package]]
        name = "new_debug_unreachable"
        version = "1.0.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e4a24736216ec316047a1fc4252e27dabb04218aa4a3f37c6e7ddbf1f9782b54"

        [[package]]
        name = "null_example"
        version = "0.0.0"
        dependencies = [
        "async-recursion",
        "azle-vm-value-derive",
        "boa_engine",
        "candid",
        "getrandom",
        "ic-cdk",
        "ic-cdk-macros",
        "serde",
        ]

        [[package]]
        name = "num-bigint"
        version = "0.4.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f93ab6289c7b344a8a9f60f88d80aa20032336fe78da341afc91c8a2341fc75f"
        dependencies = [
        "autocfg",
        "num-integer",
        "num-traits",
        "serde",
        ]

        [[package]]
        name = "num-integer"
        version = "0.1.45"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "225d3389fb3509a24c93f5c29eb6bde2586b98d9f016636dff58d7c6f7569cd9"
        dependencies = [
        "autocfg",
        "num-traits",
        ]

        [[package]]
        name = "num-traits"
        version = "0.2.15"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "578ede34cf02f8924ab9447f50c28075b4d3e5b269972345e7e0372b38c6cdcd"
        dependencies = [
        "autocfg",
        ]

        [[package]]
        name = "num_enum"
        version = "0.5.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "cf5395665662ef45796a4ff5486c5d41d29e0c09640af4c5f17fd94ee2c119c9"
        dependencies = [
        "num_enum_derive",
        ]

        [[package]]
        name = "num_enum_derive"
        version = "0.5.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3b0498641e53dd6ac1a4f22547548caa6864cc4933784319cd1775271c5a46ce"
        dependencies = [
        "proc-macro-crate",
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "once_cell"
        version = "1.14.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2f7254b99e31cad77da24b08ebf628882739a608578bb1bcdfc1f9c21260d7c0"

        [[package]]
        name = "parking_lot"
        version = "0.12.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3742b2c103b9f06bc9fff0a37ff4912935851bee6d36f3c02bcc755bcfec228f"
        dependencies = [
        "lock_api",
        "parking_lot_core",
        ]

        [[package]]
        name = "parking_lot_core"
        version = "0.9.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "09a279cbf25cb0757810394fbc1e359949b59e348145c643a939a525692e6929"
        dependencies = [
        "cfg-if",
        "libc",
        "redox_syscall",
        "smallvec",
        "windows-sys",
        ]

        [[package]]
        name = "paste"
        version = "1.0.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0c520e05135d6e763148b6426a837e239041653ba7becd2e538c076c738025fc"

        [[package]]
        name = "percent-encoding"
        version = "2.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d4fd5641d01c8f18a23da7b6fe29298ff4b55afcccdf78973b24cf3175fee32e"

        [[package]]
        name = "petgraph"
        version = "0.6.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e6d5014253a1331579ce62aa67443b4a658c5e7dd03d4bc6d302b94474888143"
        dependencies = [
        "fixedbitset",
        "indexmap",
        ]

        [[package]]
        name = "phf_generator"
        version = "0.10.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5d5285893bb5eb82e6aaf5d59ee909a06a16737a8970984dd7746ba9283498d6"
        dependencies = [
        "phf_shared",
        "rand",
        ]

        [[package]]
        name = "phf_shared"
        version = "0.10.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b6796ad771acdc0123d2a88dc428b5e38ef24456743ddb1744ed628f9815c096"
        dependencies = [
        "siphasher",
        ]

        [[package]]
        name = "pico-args"
        version = "0.4.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "db8bcd96cb740d03149cbad5518db9fd87126a10ab519c011893b1754134c468"

        [[package]]
        name = "pin-project-lite"
        version = "0.2.9"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e0a7ae3ac2f1173085d398531c705756c94a4c56843785df85a60c1a0afac116"

        [[package]]
        name = "pmutil"
        version = "0.5.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3894e5d549cccbe44afecf72922f277f603cd4bb0219c8342631ef18fffbe004"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "ppv-lite86"
        version = "0.2.16"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "eb9f9e6e233e5c4a35559a617bf40a4ec447db2e84c20b55a6f83167b7e57872"

        [[package]]
        name = "precomputed-hash"
        version = "0.1.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "925383efa346730478fb4838dbe9137d2a47675ad789c546d150a6e1dd4ab31c"

        [[package]]
        name = "pretty"
        version = "0.10.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "ad9940b913ee56ddd94aec2d3cd179dd47068236f42a1a6415ccf9d880ce2a61"
        dependencies = [
        "arrayvec",
        "typed-arena",
        ]

        [[package]]
        name = "proc-macro-crate"
        version = "1.1.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e17d47ce914bf4de440332250b0edd23ce48c005f59fab39d3335866b114f11a"
        dependencies = [
        "thiserror",
        "toml",
        ]

        [[package]]
        name = "proc-macro2"
        version = "1.0.43"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0a2ca2c61bc9f3d74d2886294ab7b9853abd9c1ad903a3ac7815c58989bb7bab"
        dependencies = [
        "unicode-ident",
        ]

        [[package]]
        name = "quote"
        version = "1.0.21"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "bbe448f377a7d6961e30f5955f9b8d106c3f5e449d493ee1b125c1d43c2b5179"
        dependencies = [
        "proc-macro2",
        ]

        [[package]]
        name = "rand"
        version = "0.8.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "34af8d1a0e25924bc5b7c43c079c942339d8f0a8b57c39049bef581b46327404"
        dependencies = [
        "libc",
        "rand_chacha",
        "rand_core",
        ]

        [[package]]
        name = "rand_chacha"
        version = "0.3.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e6c10a63a0fa32252be49d21e7709d4d4baf8d231c2dbce1eaa8141b9b127d88"
        dependencies = [
        "ppv-lite86",
        "rand_core",
        ]

        [[package]]
        name = "rand_core"
        version = "0.6.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d34f1408f55294453790c48b2f1ebbb1c5b4b7563eb1f418bcfcfdbb06ebb4e7"
        dependencies = [
        "getrandom",
        ]

        [[package]]
        name = "redox_syscall"
        version = "0.2.13"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "62f25bc4c7e55e0b0b7a1d43fb893f4fa1361d0abe38b9ce4f323c2adfe6ef42"
        dependencies = [
        "bitflags",
        ]

        [[package]]
        name = "redox_users"
        version = "0.4.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b033d837a7cf162d7993aded9304e30a83213c648b6e389db233191f891e5c2b"
        dependencies = [
        "getrandom",
        "redox_syscall",
        "thiserror",
        ]

        [[package]]
        name = "regex"
        version = "1.5.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d83f127d94bdbcda4c8cc2e50f6f84f4b611f69c902699ca385a39c3a75f9ff1"
        dependencies = [
        "aho-corasick",
        "memchr",
        "regex-syntax",
        ]

        [[package]]
        name = "regex-syntax"
        version = "0.6.26"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "49b3de9ec5dc0a3417da371aab17d729997c15010e7fd24ff707773a33bddb64"

        [[package]]
        name = "regress"
        version = "0.4.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0a92ff21fe8026ce3f2627faaf43606f0b67b014dbc9ccf027181a804f75d92e"
        dependencies = [
        "memchr",
        ]

        [[package]]
        name = "rustc-hash"
        version = "1.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "08d43f7aa6b08d49f382cde6a7982047c3426db949b1424bc4b7ec9ae12c6ce2"

        [[package]]
        name = "rustversion"
        version = "1.0.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f2cc38e8fa666e2de3c4aba7edeb5ffc5246c1c2ed0e3d17e560aeeba736b23f"

        [[package]]
        name = "ryu"
        version = "1.0.10"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f3f6f92acf49d1b98f7a81226834412ada05458b7364277387724a237f062695"

        [[package]]
        name = "ryu-js"
        version = "0.2.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6518fc26bced4d53678a22d6e423e9d8716377def84545fe328236e3af070e7f"

        [[package]]
        name = "scoped-tls"
        version = "1.0.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "ea6a9290e3c9cf0f18145ef7ffa62d68ee0bf5fcd651017e586dc7fd5da448c2"

        [[package]]
        name = "scopeguard"
        version = "1.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d29ab0c6d3fc0ee92fe66e2d99f700eab17a8d57d1c1d3b748380fb20baa78cd"

        [[package]]
        name = "serde"
        version = "1.0.137"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "61ea8d54c77f8315140a05f4c7237403bf38b72704d031543aa1d16abbf517d1"
        dependencies = [
        "serde_derive",
        ]

        [[package]]
        name = "serde_bytes"
        version = "0.11.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "212e73464ebcde48d723aa02eb270ba62eff38a9b732df31f33f1b4e145f3a54"
        dependencies = [
        "serde",
        ]

        [[package]]
        name = "serde_derive"
        version = "1.0.137"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1f26faba0c3959972377d3b2d306ee9f71faee9714294e41bb777f83f88578be"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "serde_json"
        version = "1.0.81"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "9b7ce2b32a1aed03c558dc61a5cd328f15aff2dbc17daad8fb8af04d2100e15c"
        dependencies = [
        "itoa",
        "ryu",
        "serde",
        ]

        [[package]]
        name = "serde_tokenstream"
        version = "0.1.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d6deb15c3a535e81438110111d90168d91721652f502abb147f31cde129f683d"
        dependencies = [
        "proc-macro2",
        "serde",
        "syn",
        ]

        [[package]]
        name = "sha2"
        version = "0.10.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "cf9db03534dff993187064c4e0c05a5708d2a9728ace9a8959b77bedf415dac5"
        dependencies = [
        "cfg-if",
        "cpufeatures",
        "digest",
        ]

        [[package]]
        name = "siphasher"
        version = "0.3.10"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "7bd3e3206899af3f8b12af284fafc038cc1dc2b41d1b89dd17297221c5d225de"

        [[package]]
        name = "smallvec"
        version = "1.8.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f2dd574626839106c320a323308629dcb1acfc96e32a8cba364ddc61ac23ee83"
        dependencies = [
        "serde",
        ]

        [[package]]
        name = "stable_deref_trait"
        version = "1.2.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a8f112729512f8e442d81f95a8a7ddf2b7c6b8a1a6f509a95864142b30cab2d3"

        [[package]]
        name = "static_assertions"
        version = "1.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a2eb9349b6444b326872e140eb1cf5e7c522154d69e7a0ffb0fb81c06b37543f"

        [[package]]
        name = "string-interner"
        version = "0.14.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "91e2531d8525b29b514d25e275a43581320d587b86db302b9a7e464bac579648"
        dependencies = [
        "cfg-if",
        "hashbrown 0.11.2",
        "serde",
        ]

        [[package]]
        name = "string_cache"
        version = "0.8.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "213494b7a2b503146286049378ce02b482200519accc31872ee8be91fa820a08"
        dependencies = [
        "new_debug_unreachable",
        "once_cell",
        "parking_lot",
        "phf_shared",
        "precomputed-hash",
        "serde",
        ]

        [[package]]
        name = "string_cache_codegen"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6bb30289b722be4ff74a408c3cc27edeaad656e06cb1fe8fa9231fa59c728988"
        dependencies = [
        "phf_generator",
        "phf_shared",
        "proc-macro2",
        "quote",
        ]

        [[package]]
        name = "string_enum"
        version = "0.3.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f584cc881e9e5f1fd6bf827b0444aa94c30d8fe6378cf241071b5f5700b2871f"
        dependencies = [
        "pmutil",
        "proc-macro2",
        "quote",
        "swc_macros_common",
        "syn",
        ]

        [[package]]
        name = "strsim"
        version = "0.10.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "73473c0e59e6d5812c5dfe2a064a6444949f089e20eec9a2e5506596494e4623"

        [[package]]
        name = "swc_atoms"
        version = "0.4.10"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "8fb43a79c8affc20f5d52b7db093399585ce87674427adc60843dbc8ec242608"
        dependencies = [
        "once_cell",
        "rustc-hash",
        "serde",
        "string_cache",
        "string_cache_codegen",
        ]

        [[package]]
        name = "swc_common"
        version = "0.27.13"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "cba38a2f1291fcf3f78f357802b8cec72ecf5e95808e9d937783e60cd3570b93"
        dependencies = [
        "ahash",
        "ast_node",
        "better_scoped_tls",
        "cfg-if",
        "debug_unreachable",
        "either",
        "from_variant",
        "num-bigint",
        "once_cell",
        "rustc-hash",
        "serde",
        "siphasher",
        "string_cache",
        "swc_atoms",
        "swc_eq_ignore_macros",
        "swc_visit",
        "tracing",
        "unicode-width",
        "url",
        ]

        [[package]]
        name = "swc_ecma_ast"
        version = "0.90.15"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "571989e199094be58d107e032c0b868f7b04a59a238e0a31bc9df8faf537dcd3"
        dependencies = [
        "bitflags",
        "is-macro",
        "num-bigint",
        "scoped-tls",
        "serde",
        "string_enum",
        "swc_atoms",
        "swc_common",
        "unicode-id",
        ]

        [[package]]
        name = "swc_ecma_parser"
        version = "0.117.7"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d88ffca99514bc8bfa8bf4f2d74fb673caa844e521b55035b8cfd10472b801d4"
        dependencies = [
        "either",
        "enum_kind",
        "lexical",
        "num-bigint",
        "serde",
        "smallvec",
        "swc_atoms",
        "swc_common",
        "swc_ecma_ast",
        "tracing",
        "typed-arena",
        ]

        [[package]]
        name = "swc_eq_ignore_macros"
        version = "0.1.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0c20468634668c2bbab581947bb8c75c97158d5a6959f4ba33df20983b20b4f6"
        dependencies = [
        "pmutil",
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "swc_macros_common"
        version = "0.3.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a4be988307882648d9bc7c71a6a73322b7520ef0211e920489a98f8391d8caa2"
        dependencies = [
        "pmutil",
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "swc_visit"
        version = "0.5.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "b754ef01f2614eb469fd5215789bfd319566a3bf1033056d7a7bfb5a3c9a69f5"
        dependencies = [
        "either",
        "swc_visit_macros",
        ]

        [[package]]
        name = "swc_visit_macros"
        version = "0.5.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "c230bcd129d1fbcd1decd8b43cccd613fda11c895f7c04d6c966231dbc1959af"
        dependencies = [
        "Inflector",
        "pmutil",
        "proc-macro2",
        "quote",
        "swc_macros_common",
        "syn",
        ]

        [[package]]
        name = "syn"
        version = "1.0.99"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "58dbef6ec655055e20b86b15a8cc6d439cca19b667537ac6a1369572d151ab13"
        dependencies = [
        "proc-macro2",
        "quote",
        "unicode-ident",
        ]

        [[package]]
        name = "synstructure"
        version = "0.12.6"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "f36bdaa60a83aca3921b5259d5400cbf5e90fc51931376a9bd4a0eb79aa7210f"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        "unicode-xid",
        ]

        [[package]]
        name = "tap"
        version = "1.0.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "55937e1799185b12863d447f42597ed69d9928686b8d88a1df17376a097d8369"

        [[package]]
        name = "term"
        version = "0.7.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "c59df8ac95d96ff9bede18eb7300b0fda5e5d8d90960e76f8e14ae765eedbf1f"
        dependencies = [
        "dirs-next",
        "rustversion",
        "winapi",
        ]

        [[package]]
        name = "termcolor"
        version = "1.1.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "bab24d30b911b2376f3a13cc2cd443142f0c81dda04c118693e35b3835757755"
        dependencies = [
        "winapi-util",
        ]

        [[package]]
        name = "thiserror"
        version = "1.0.31"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "bd829fe32373d27f76265620b5309d0340cb8550f523c1dda251d6298069069a"
        dependencies = [
        "thiserror-impl",
        ]

        [[package]]
        name = "thiserror-impl"
        version = "1.0.31"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0396bc89e626244658bef819e22d0cc459e795a5ebe878e6ec336d1674a8d79a"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "time"
        version = "0.1.44"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6db9e6914ab8b1ae1c260a4ae7a49b6c5611b40328a735b21862567685e73255"
        dependencies = [
        "libc",
        "wasi 0.10.0+wasi-snapshot-preview1",
        "winapi",
        ]

        [[package]]
        name = "tiny-keccak"
        version = "2.0.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2c9d3793400a45f954c52e73d068316d76b6f4e36977e3fcebb13a2721e80237"
        dependencies = [
        "crunchy",
        ]

        [[package]]
        name = "tinystr"
        version = "0.4.12"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "954af207a9e273b644c2c0d29d79d7cba9c22a66e633535d0d0d3712d7e50563"
        dependencies = [
        "serde",
        "tinystr-macros",
        "tinystr-raw",
        ]

        [[package]]
        name = "tinystr-macros"
        version = "0.2.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5f65be51117c325c2b58eec529be7a0857d11527a9029973b58810a4c63e77a6"
        dependencies = [
        "tinystr-raw",
        ]

        [[package]]
        name = "tinystr-raw"
        version = "0.1.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "10f87ef8b0485e4efff5cac95608adc3251e412fef6039ecd56c5618c8003895"

        [[package]]
        name = "tinyvec"
        version = "1.6.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "87cc5ceb3875bb20c2890005a4e226a4651264a5c75edb2421b52861a0a0cb50"
        dependencies = [
        "tinyvec_macros",
        ]

        [[package]]
        name = "tinyvec_macros"
        version = "0.1.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "cda74da7e1a664f795bb1f8a87ec406fb89a02522cf6e50620d016add6dbbf5c"

        [[package]]
        name = "toml"
        version = "0.5.9"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "8d82e1a7758622a465f8cee077614c73484dac5b836c02ff6a40d5d1010324d7"
        dependencies = [
        "serde",
        ]

        [[package]]
        name = "tracing"
        version = "0.1.36"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "2fce9567bd60a67d08a16488756721ba392f24f29006402881e43b19aac64307"
        dependencies = [
        "cfg-if",
        "pin-project-lite",
        "tracing-attributes",
        "tracing-core",
        ]

        [[package]]
        name = "tracing-attributes"
        version = "0.1.22"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "11c75893af559bc8e10716548bdef5cb2b983f8e637db9d0e15126b61b484ee2"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "tracing-core"
        version = "0.1.29"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5aeea4303076558a00714b823f9ad67d58a3bbda1df83d8827d21193156e22f7"
        dependencies = [
        "once_cell",
        ]

        [[package]]
        name = "typed-arena"
        version = "2.0.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0685c84d5d54d1c26f7d3eb96cd41550adb97baed141a761cf335d3d33bcd0ae"

        [[package]]
        name = "typenum"
        version = "1.15.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "dcf81ac59edc17cc8697ff311e8f5ef2d99fcbd9817b34cec66f90b6c3dfd987"

        [[package]]
        name = "unicode-bidi"
        version = "0.3.8"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "099b7128301d285f79ddd55b9a83d5e6b9e97c92e0ea0daebee7263e932de992"

        [[package]]
        name = "unicode-general-category"
        version = "0.5.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1218098468b8085b19a2824104c70d976491d247ce194bbd9dc77181150cdfd6"

        [[package]]
        name = "unicode-id"
        version = "0.3.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "69fe8d9274f490a36442acb4edfd0c4e473fdfc6a8b5cd32f28a0235761aedbe"

        [[package]]
        name = "unicode-ident"
        version = "1.0.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5bd2fe26506023ed7b5e1e315add59d6f584c621d037f9368fea9cfb988f368c"

        [[package]]
        name = "unicode-normalization"
        version = "0.1.19"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "d54590932941a9e9266f0832deed84ebe1bf2e4c9e4a3554d393d18f5e854bf9"
        dependencies = [
        "tinyvec",
        ]

        [[package]]
        name = "unicode-width"
        version = "0.1.9"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "3ed742d4ea2bd1176e236172c8429aaf54486e7ac098db29ffe6529e0ce50973"

        [[package]]
        name = "unicode-xid"
        version = "0.2.3"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "957e51f3646910546462e67d5f7599b9e4fb8acdd304b087a6494730f9eebf04"

        [[package]]
        name = "unreachable"
        version = "0.1.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1f2ae5ddb18e1c92664717616dd9549dde73f539f01bd7b77c2edb2446bdff91"
        dependencies = [
        "void",
        ]

        [[package]]
        name = "url"
        version = "2.2.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "a507c383b2d33b5fc35d1861e77e6b383d158b2da5e14fe51b83dfedf6fd578c"
        dependencies = [
        "form_urlencoded",
        "idna",
        "matches",
        "percent-encoding",
        ]

        [[package]]
        name = "uuid"
        version = "1.1.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "dd6469f4314d5f1ffec476e05f17cc9a78bc7a27a6a857842170bdf8d6f98d2f"
        dependencies = [
        "getrandom",
        "rand",
        "uuid-macro-internal",
        ]

        [[package]]
        name = "uuid-macro-internal"
        version = "1.1.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "548f7181a5990efa50237abb7ebca410828b57a8955993334679f8b50b35c97d"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        ]

        [[package]]
        name = "version_check"
        version = "0.9.4"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "49874b5167b65d7193b8aba1567f5c7d93d001cafc34600cee003eda787e483f"

        [[package]]
        name = "void"
        version = "1.0.2"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "6a02e4885ed3bc0f2de90ea6dd45ebcbb66dacffe03547fadbb0eeae2770887d"

        [[package]]
        name = "wasi"
        version = "0.10.0+wasi-snapshot-preview1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "1a143597ca7c7793eff794def352d41792a93c481eb1042423ff7ff72ba2c31f"

        [[package]]
        name = "wasi"
        version = "0.11.0+wasi-snapshot-preview1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "9c8d87e72b64a3b4db28d11ce29237c246188f4f51057d65a7eab63b7987e423"

        [[package]]
        name = "winapi"
        version = "0.3.9"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "5c839a674fcd7a98952e593242ea400abe93992746761e38641405d28b00f419"
        dependencies = [
        "winapi-i686-pc-windows-gnu",
        "winapi-x86_64-pc-windows-gnu",
        ]

        [[package]]
        name = "winapi-i686-pc-windows-gnu"
        version = "0.4.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "ac3b87c63620426dd9b991e5ce0329eff545bccbbb34f3be09ff6fb6ab51b7b6"

        [[package]]
        name = "winapi-util"
        version = "0.1.5"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "70ec6ce85bb158151cae5e5c87f95a8e97d2c0c4b001223f33a334e3ce5de178"
        dependencies = [
        "winapi",
        ]

        [[package]]
        name = "winapi-x86_64-pc-windows-gnu"
        version = "0.4.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "712e227841d057c1ee1cd2fb22fa7e5a5461ae8e48fa2ca79ec42cfc1931183f"

        [[package]]
        name = "windows-sys"
        version = "0.36.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "ea04155a16a59f9eab786fe12a4a450e75cdb175f9e0d80da1e17db09f55b8d2"
        dependencies = [
        "windows_aarch64_msvc",
        "windows_i686_gnu",
        "windows_i686_msvc",
        "windows_x86_64_gnu",
        "windows_x86_64_msvc",
        ]

        [[package]]
        name = "windows_aarch64_msvc"
        version = "0.36.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "9bb8c3fd39ade2d67e9874ac4f3db21f0d710bee00fe7cab16949ec184eeaa47"

        [[package]]
        name = "windows_i686_gnu"
        version = "0.36.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "180e6ccf01daf4c426b846dfc66db1fc518f074baa793aa7d9b9aaeffad6a3b6"

        [[package]]
        name = "windows_i686_msvc"
        version = "0.36.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "e2e7917148b2812d1eeafaeb22a97e4813dfa60a3f8f78ebe204bcc88f12f024"

        [[package]]
        name = "windows_x86_64_gnu"
        version = "0.36.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "4dcd171b8776c41b97521e5da127a2d86ad280114807d0b2ab1e462bc764d9e1"

        [[package]]
        name = "windows_x86_64_msvc"
        version = "0.36.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "c811ca4a8c853ef420abd8592ba53ddbbac90410fab6903b3e79972a631f7680"

        [[package]]
        name = "writeable"
        version = "0.3.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "063dba1bbc4c97dfb9b9e57cf4bcf3bac1ebfd2ef60bc609fc08dcd0228c3de5"

        [[package]]
        name = "yoke"
        version = "0.4.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "0a32bb18a3c6c8c2726ad84153babdf8f5336a7d5551a9177a79d006f0ce789f"
        dependencies = [
        "serde",
        "stable_deref_trait",
        "yoke-derive",
        ]

        [[package]]
        name = "yoke-derive"
        version = "0.4.1"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "77b70271795226acc88b2dfe0651e91e6402b64b1887c60722e04be55ff15941"
        dependencies = [
        "proc-macro2",
        "quote",
        "syn",
        "synstructure",
        ]

        [[package]]
        name = "zerovec"
        version = "0.6.0"
        source = "registry+https://github.com/rust-lang/crates.io-index"
        checksum = "4ae959aabd79f0f4dec802b32404cf41776d03757dd5b6e815d3d1ca076fa939"
        dependencies = [
        "serde",
        "yoke",
        ]

    `;
}
